using GraphicsManager.Enums;
using GraphicsManager.Objects;
using Luski.net.JsonTypes;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common;
using OpenTK.Windowing.GraphicsLibraryFramework;

namespace Luski.GUI.MainScreen.UI;

public class AddFriendPage : FlowLayout
{
    private MainScreen Screen;
    private Textbox Input;
    
    public AddFriendPage(MainScreen Parent)
    {
        this.Screen = Parent;
        Size = new((int)(200 * Globals.Settings.Scale), (int)(48 * Globals.Settings.Scale));
        BackgroundColor = new(45,45,45,255);
        Controls.Add(Input = new Textbox() { Anchor = ObjectAnchor.Left | ObjectAnchor.Top | ObjectAnchor.Right, Location = new((int)(7.5 * Globals.Settings.Scale)), Size = new((int)(185 * Globals.Settings.Scale),(int)(20 * Globals.Settings.Scale)), InsideColor = new(28,28,28,255), BorderColor = Color4.DarkCyan });
        Input.KeyPress += InputOnKeyPress;
    }

    private Task InputOnKeyPress(KeyboardKeyEventArgs arg)
    {
        
        if (Input.BorderColor == Color4.Red) Input.BorderColor = Color4.DarkCyan;
        if (arg.Key != Keys.Enter && arg.Key != Keys.KeyPadEnter) return Task.CompletedTask;
        if (!long.TryParse(Input.Text, out long code))
        {
            Input.BorderColor = Color4.Red;
            return Task.CompletedTask;
        }
        SocketRemoteUser? result = Globals.Luski.SendFriendRequest(code, CancellationToken.None).Result;
        if (result is null) Input.BorderColor = Color4.Red;
        else
        {
            if (result.Channel is null)
                Screen.AddFriendRequest(result);
            else
                Screen.AddFriend(result);
        }
        return Task.CompletedTask;
    }
}