using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.net.Enums;
using Luski.net.JsonTypes;
using OpenTK.Mathematics;

namespace Luski.GUI.MainScreen.UI;

public class FriendRequest : UserControl
{
    private MainScreen Screen;
    public SocketRemoteUser User;
    public Rectangle Accept, Reject;
    
    public FriendRequest(MainScreen Parent, SocketRemoteUser User)
    {
        this.User = User;
        this.Screen = Parent;
        Size = new((int)(200 * Globals.Settings.Scale), (int)(48* Globals.Settings.Scale));
        BackgroundColor = new(34, 34, 34, 255);
        Controls.Add(new Rectangle(Globals.ms.TextureManager.AddTexture(User.GetAvatar(CancellationToken.None).Result)) { Location = new((int)(7.5 * Globals.Settings.Scale),(int)(8.5 * Globals.Settings.Scale)), Size = new((int)(29 * Globals.Settings.Scale))});
        Controls.Add(new Label() { Font = Globals.DefaultFont, Text = User.Username, Location = new((int)(38.5 * Globals.Settings.Scale),(int)(10 * Globals.Settings.Scale))});
        Controls.Add(new Label() { Font = Globals.DefaultFont, Text = User.FriendStatus.ToString(), Location = new((int)(38.5 * Globals.Settings.Scale),(int)(24 * Globals.Settings.Scale))});
        if (User.FriendStatus == FriendStatus.PendingIn)
        {
            Controls.Add(Accept = new Rectangle()
            {
                Anchor = ObjectAnchor.Top | ObjectAnchor.Right, BackgroundColor = Color4.Green, Location = new((int)(127 * Globals.Settings.Scale), (int)(8.5 * Globals.Settings.Scale)),
                Size = new((int)(29 * Globals.Settings.Scale))
            });
            Controls.Add(Reject = new Rectangle()
            {
                Anchor = ObjectAnchor.Top | ObjectAnchor.Right, BackgroundColor = Color4.Red, Location = new((int)(163.5 * Globals.Settings.Scale), (int)(8.5 * Globals.Settings.Scale)),
                Size = new((int)(29 * Globals.Settings.Scale))
            });
            Accept.Clicked += AcceptOnClicked;
            Reject.Clicked += RejectOnClicked;
        }
    }

    private Task RejectOnClicked(IRenderObject arg)
    {
        _ = Globals.Luski.SendFriendResult(User.Id, false, CancellationToken.None).Result;
        Screen.RemoveFriendRequest(User);
        return Task.CompletedTask;
    }

    private Task AcceptOnClicked(IRenderObject arg)
    {
        Screen.AddFriend(Globals.Luski.SendFriendResult(User.Id, true, CancellationToken.None).Result);
        Screen.RemoveFriendRequest(User);
        return Task.CompletedTask;
    }
}