using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using Luski.net;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common;
using OpenTK.Windowing.GraphicsLibraryFramework;

namespace Luski.GUI.StartPage.UI;

public class CreateAccount : UserControl
{
    private RoundedButton button;
    private Textbox Password, Email, Username;
    private Rectangle rec;
    private string pfp = null;
    
    public event Func<Task> ChangeToApp;
    
    public CreateAccount()
    {
        Size = new((int)(240.5*Globals.Settings.Scale), (int)(419*Globals.Settings.Scale));
        Controls.Add(new Rectangle(Globals.LuskiTexture) { Location = new((int)(51.5 * Globals.Settings.Scale), (int)(4 * Globals.Settings.Scale)), Size = new((int)(138*Globals.Settings.Scale), (int)(144.5*Globals.Settings.Scale))});
        Label t;
        Controls.Add(t=new Label()   { Scale = 1.6f, Font = Globals.DefaultFont, Location = new((int)(85*Globals.Settings.Scale),(int)(153*Globals.Settings.Scale)), Text = "Luski", Color = new(243, 119, 53, 255) });
        t.Location = new((Size.X / 2) - (t.Size.X / 2), t.Location.Y);
        Controls.Add(Email = new Textbox() { WatermarkFont = Globals.DefaultFont, WatermarkText = "Email", Font = Globals.DefaultFont,  Location = new((int)(20.5 * Globals.Settings.Scale),(int)(215.5 * Globals.Settings.Scale)), Size = new((int)(200.5 * Globals.Settings.Scale),(int)(20.5 * Globals.Settings.Scale)), InsideColor = new(28,28,28,255), BorderColor = Color4.DarkCyan });
        Controls.Add(Password = new Textbox() { WatermarkFont = Globals.DefaultFont, WatermarkText = "Password", Font = Globals.DefaultFont, PasswordChar = '●', Location = new(Email.Location.X,(int)(265.5 * Globals.Settings.Scale)), Size = new(Email.Size.X, Email.Location.X), InsideColor = new(28, 28, 28, 255), BorderColor = Color4.DarkCyan });
        Controls.Add(Username = new Textbox() { WatermarkFont = Globals.DefaultFont, WatermarkText = "Username", Font = Globals.DefaultFont, Location = new(Email.Location.X,(int)(308 * Globals.Settings.Scale)), Size = new((int)(150 * Globals.Settings.Scale), Email.Location.X), InsideColor = new(28, 28, 28, 255), BorderColor = Color4.DarkCyan });
        Controls.Add(button = new() { Font = Globals.DefaultFont, Text = "Create Account", Location = new(Email.Location.X, (int)(350 * Globals.Settings.Scale)), Size = new((int)(200.5 * Globals.Settings.Scale), (int)(35.5 * Globals.Settings.Scale)), InsideColor = new(28, 28, 28, 255), BorderColor = Color4.DarkCyan });
        Controls.Add(rec = new Rectangle(){ Location = new((int)(175 * Globals.Settings.Scale), (int)(292.5 * Globals.Settings.Scale)), Size = new((int)(50 * Globals.Settings.Scale)), BackgroundColor = Color4.Red});
        Password.KeyPress += PasswordOnKeyPress;
        Email.KeyPress += EmailOnKeyPress;
        button.Clicked += ButtonOnClicked;
        Username.KeyPress += UsernameOnKeyPress;
        rec.FilesDroped += RecOnFilesDroped;
    }

    private Task UsernameOnKeyPress(KeyboardKeyEventArgs arg)
    {
        if (arg.Key != Keys.Enter && arg.Key != Keys.KeyPadEnter && arg.Key != Keys.Tab) return Task.CompletedTask;
        if (arg.Key == Keys.Tab && arg.Shift) { Password.Focus(); return Task.CompletedTask; } 
        if (arg.Key == Keys.Enter || arg.Key == Keys.KeyPadEnter) ButtonOnClicked(Username);
        return Task.CompletedTask;
    }

    private Task RecOnFilesDroped(string[] arg)
    {
        if (!arg[0].ToLower().EndsWith("png")) return Task.CompletedTask;
        Controls.Remove(rec);
        pfp = arg[0];
        rec = new(Globals.ms.TextureManager.AddTexture(File.ReadAllBytes(arg[0]))){ Location = new(350, 585), Size = new(100) };
        Controls.Add(rec);
        Window!.DrawFrame();
        return Task.CompletedTask;
    }

    private bool tab;
    private Task EmailOnKeyPress(KeyboardKeyEventArgs arg)
    {
        if (arg.Key != Keys.Tab) return Task.CompletedTask;
        if (arg.Key == Keys.Tab){ Password.Focus(); tab = true;}
        return Task.CompletedTask;
    }

    private Task PasswordOnKeyPress(KeyboardKeyEventArgs arg)
    {
        if (tab)
        {
            tab = false;
            return Task.CompletedTask;
        };
        if (arg.Key != Keys.Enter && arg.Key != Keys.KeyPadEnter && arg.Key != Keys.Tab) return Task.CompletedTask;
        if (arg.Key == Keys.Tab && arg.Shift) { Email.Focus(); return Task.CompletedTask; }
        if (arg.Key == Keys.Tab ){ Username.Focus(); return Task.CompletedTask; }
        return Task.CompletedTask;
    }

    private Task ButtonOnClicked(IRenderObject arg)
    {
        try
        {
            string[] arr = new string[]
            {
                Email.Text,
                Password.Text,
                Username.Text,
                pfp
            };
            if (arr.Any(s => string.IsNullOrEmpty(s))) return Task.CompletedTask;
            Globals.Luski = Server.CreateAccount(Email.Text, Password.Text, Username.Text, pfp, CancellationToken.None, Globals.UpdaterSettings.Branch);
            ChangeToApp.Invoke();
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
        }
        return Task.CompletedTask;
    }
}