﻿using System.Diagnostics;
using System.Reflection;
using System.Security.Cryptography;
using Luski;
using Luski.Clesses;
using Luski.GUI.MainScreen;
using Luski.net;
using OpenTK.Windowing.Common.Input;
using SixLabors.ImageSharp.PixelFormats;

try
{
    
    Globals.Settings = Globals.GetSettings(Path.Combine(Globals.LuskiPath, "Settings.json"), SettingsContext.Default.Settings);
    Globals.UpdaterSettings = Globals.GetSettings(Path.Combine(Globals.LuskiPath, "UpdaterSettings.json"), UpdaterSettingsContext.Default.UpdaterSettings);
    
    Assembly assembly = Assembly.GetExecutingAssembly();
    Stream? resource_stream = assembly.GetManifestResourceStream($"Luski.Resources.Textures.Luski.png");
    MemoryStream ms = new();
    resource_stream?.CopyTo(ms);
    var Logo = SixLabors.ImageSharp.Image.Load<Rgba32>(ms.ToArray());

    var pixels = new List<byte>(4 * Logo.Width * Logo.Height);

    for (int y = 0; y < Logo.Height; y++)
    {
        var row = Logo.GetPixelRowSpan(y);

        for (int x = 0; x < Logo.Width; x++)
        {
            pixels.Add(row[x].R);
            pixels.Add(row[x].G);
            pixels.Add(row[x].B);
            pixels.Add(row[x].A);
        }
    }
    Globals.Icon = new WindowIcon(new Image(Logo.Width, Logo.Height, pixels.ToArray()));
    Globals.ms = new MainScreen();
    Globals.ms.StartRender();
    Globals.ms.Dispose();
}
catch (Exception ex)
{
    Console.WriteLine(ex);
}

if (Globals.Download)
{
    Console.WriteLine("Auto Update Starting");
    List<string> arguments = new List<string>
    {
        "--process",
        Process.GetCurrentProcess().ProcessName,
        "--remotedirectory",
        "Luski",
        "--localdirectory",
        AppDomain.CurrentDomain.BaseDirectory,
        "--branch",
        Globals.UpdaterSettings.Branch.ToString(),
        "--selfcontained",
        Globals.UpdaterSettings.SelfContained.ToString().ToLower(),
        "--platform",
        Globals.UpdaterSettings.Platform,
        "--setconfig",
        Path.Combine(Globals.LuskiPath, "UpdaterSettings.json")
    };
    if (Globals.UpdaterSettings.AutoLaunch)
    {
        arguments.Add("--dll");
        arguments.Add(AppDomain.CurrentDomain.FriendlyName);
    }
    Process p = new();
    p.StartInfo.FileName = Globals.UpdaterSettings.Updater;
    p.StartInfo.WorkingDirectory = new FileInfo(Globals.UpdaterSettings.Updater!).Directory!.FullName;
    p.StartInfo.Arguments = $"\"{string.Join("\" \"", arguments)}\"";
    p.Start();
    Environment.Exit(0);
}